import { atom } from 'jotai';
import { QueryClient, QueryObserver } from 'react-query';

const queryClientAtom = atom(null);
const getQueryClientAtom = atom((get) => get(queryClientAtom) || new QueryClient());

function atomWithQuery(createQuery, equalityFn = Object.is) {
  const queryDataAtom = atom((get) => {
    const queryClient = get(getQueryClientAtom);
    const options = typeof createQuery === "function" ? createQuery(get) : createQuery;
    let resolve = null;
    const dataAtom = atom(new Promise((r) => {
      resolve = r;
    }));
    let setData = () => {
      throw new Error("atomWithQuery: setting data without mount");
    };
    let prevData = null;
    const listener = (result) => {
      if (result.data === void 0 || prevData !== null && equalityFn(prevData, result.data)) {
        return;
      }
      prevData = result.data;
      if (resolve) {
        resolve(result.data);
        resolve = null;
      } else {
        setData(result.data);
      }
    };
    const defaultedOptions = queryClient.defaultQueryObserverOptions(options);
    if (typeof defaultedOptions.staleTime !== "number") {
      defaultedOptions.staleTime = 1e3;
    }
    const observer = new QueryObserver(queryClient, defaultedOptions);
    observer.fetchOptimistic(defaultedOptions).then(listener).catch(() => {
    });
    dataAtom.onMount = (update) => {
      setData = update;
      const unsubscribe = observer.subscribe(listener);
      return unsubscribe;
    };
    return { dataAtom, options };
  }, (get, set, action) => {
    var _a;
    switch (action.type) {
      case "refetch": {
        const { dataAtom, options } = get(queryDataAtom);
        set(dataAtom, new Promise(() => {
        }));
        const queryClient = get(getQueryClientAtom);
        (_a = queryClient.getQueryCache().find(options.queryKey)) == null ? void 0 : _a.reset();
        const p = queryClient.refetchQueries(options.queryKey);
        return p;
      }
    }
  });
  const queryAtom = atom((get) => {
    const { dataAtom } = get(queryDataAtom);
    return get(dataAtom);
  }, (_get, set, action) => set(queryDataAtom, action));
  return queryAtom;
}

export { atomWithQuery, queryClientAtom };
