'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var jotai = require('jotai');
var reactQuery = require('react-query');

var queryClientAtom = jotai.atom(null);
var getQueryClientAtom = jotai.atom(function (get) {
  return get(queryClientAtom) || new reactQuery.QueryClient();
});

function atomWithQuery(createQuery, equalityFn) {
  if (equalityFn === void 0) {
    equalityFn = Object.is;
  }

  var queryDataAtom = jotai.atom(function (get) {
    var queryClient = get(getQueryClientAtom);
    var options = typeof createQuery === 'function' ? createQuery(get) : createQuery;
    var resolve = null;
    var dataAtom = jotai.atom(new Promise(function (r) {
      resolve = r;
    }));

    var setData = function setData() {
      throw new Error('atomWithQuery: setting data without mount');
    };

    var prevData = null;

    var listener = function listener(result) {
      if (result.data === undefined || prevData !== null && equalityFn(prevData, result.data)) {
        return;
      }

      prevData = result.data;

      if (resolve) {
        resolve(result.data);
        resolve = null;
      } else {
        setData(result.data);
      }
    };

    var defaultedOptions = queryClient.defaultQueryObserverOptions(options);

    if (typeof defaultedOptions.staleTime !== 'number') {
      defaultedOptions.staleTime = 1000;
    }

    var observer = new reactQuery.QueryObserver(queryClient, defaultedOptions);
    observer.fetchOptimistic(defaultedOptions).then(listener).catch(function () {});

    dataAtom.onMount = function (update) {
      setData = update;
      var unsubscribe = observer.subscribe(listener);
      return unsubscribe;
    };

    return {
      dataAtom: dataAtom,
      options: options
    };
  }, function (get, set, action) {
    switch (action.type) {
      case 'refetch':
        {
          var _queryClient$getQuery;

          var _get2 = get(queryDataAtom),
              dataAtom = _get2.dataAtom,
              options = _get2.options;

          set(dataAtom, new Promise(function () {}));
          var queryClient = get(getQueryClientAtom);
          (_queryClient$getQuery = queryClient.getQueryCache().find(options.queryKey)) == null ? void 0 : _queryClient$getQuery.reset();
          var p = queryClient.refetchQueries(options.queryKey);
          return p;
        }
    }
  });
  var queryAtom = jotai.atom(function (get) {
    var _get3 = get(queryDataAtom),
        dataAtom = _get3.dataAtom;

    return get(dataAtom);
  }, function (_get, set, action) {
    return set(queryDataAtom, action);
  });
  return queryAtom;
}

exports.atomWithQuery = atomWithQuery;
exports.queryClientAtom = queryClientAtom;
